@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-# $Id: test_functional.py 6311 2010-04-23 13:09:58Z milde $
+# $Id: test_functional.py 7039 2011-05-24 12:28:18Z milde $
 # Author: Lea Wiemann <LeWiemann@gmail.com>
 # Copyright: This module has been placed in the public domain.
 
@@ -163,19 +163,27 @@ expected output and check it in:
             try:
                 output = output.decode(output_encoding)
             except UnicodeDecodeError:
-                # failsafe (default for latex2e writer)
+                # failsafe
                 output = output.decode('latin1', 'replace')
+        # Normalize line endings:
+        output = '\n'.join(output.splitlines())
         # Get the expected output *after* writing the actual output.
         no_expected = self.no_expected_template % {
             'exp': expected_path, 'out': params['destination_path']}
         self.assert_(os.access(expected_path, os.R_OK), no_expected)
-        f = open(expected_path, 'rb')
-        expected = f.read()
+        if sys.version_info < (3,0):
+            f = open(expected_path, 'r')
+        else: # samples are UTF8 encoded. 'rb' leads to errors with Python 3!
+            f = open(expected_path, 'r', encoding='utf-8')
+        f = open(expected_path, 'r')
+        # Normalize line endings:
+        expected = '\n'.join(f.read().splitlines())
         f.close()
-        try:
-            expected = expected.decode(output_encoding)
-        except UnicodeDecodeError:
-            expected = expected.decode('latin1', 'replace')
+        if sys.version_info < (3,0):
+            try:
+                expected = expected.decode(output_encoding)
+            except UnicodeDecodeError:
+                expected = expected.decode('latin1', 'replace')
 
         diff = self.expected_output_differs_template % {
             'exp': expected_path, 'out': params['destination_path']}
@@ -186,7 +194,7 @@ expected output and check it in:
                 expected.splitlines(True), output.splitlines(True),
                 expected_path, params['destination_path']))
             if sys.version_info < (3,0):
-                diff = diff.encode('ascii', 'replace')
+                diff = diff.encode(sys.stderr.encoding or 'ascii', 'replace')
             print >>sys.stderr, '\n%s:' % (self,)
             print >>sys.stderr, diff
             raise
